# BSVARsampleA0.R
#
# Draw posterrior samples of structural A0 coefficients for the
# various models
#
# Brandt and Freeman BVAR 2 paper -- AJPS
#
# Patrick Brandt
# pbrandt@utdallas.edu
#
######################################################################
# Begun: 08/16/2005
#
######################################################################

# Load libraries
library(MSBVAR)  # Brandt BVAR / VAR library

# Load Rmpi and snow for parallelization
library(Rmpi)
library(snow)

# Load the fitted model results and posterior modes
load("BSVARfit.RData")

# Set up the parallelization features / cluster
cl <- makeCluster(spec=2)

# Seed the RNG streams on each node of the cluster
clusterSetupRNG(cl, seed=c(16243, 1699))

####################################################################
# Posterior MCMC sampling:
# 1) Draw values from the posterior of A0
# 2) Find the fit measures and do convergence diagnostics  Do the
# convergence diagnostics on the inverse of A0, which is the
# quantity of interest.
####################################################################


####################################################################
# Running tally model
####################################################################

print(system.time(running.tally.A0.parallel <- clusterApply(cl, 1:2,
                                                            parallel.model,
                                                            fit.model=running.tally.model,
                                                            N1, N2, thin.iter)))

# Extracts the free parameters in the A0 matrix for analysis using
# convergence diagnostic tools

A0.free <- A0.chains2mcmc.list(running.tally.A0.parallel)

# Convergence diagnostics

print(geweke.diag(A0.free))   # Geweke diagnostic
print(effectiveSize(A0.free))
print(gelman.diag(A0.free))

# Get the sign pattern for the A0 so we can normalize
signlist <- sign(apply(rbind(A0.free[[1]], A0.free[[2]]), 2, median))

# Save te A0 data for more analysis later
save(A0.free, file="A0free-runningtally.RData", compress=T)
rm(A0.free); gc()

# Save the A0 object
running.tally.A0 <- A0parallel2A0obj(running.tally.A0.parallel)

# Save parallel chain
save(running.tally.model,
     running.tally.A0.parallel,
     signlist,
     file="A0parallel-runningtally.RData", compress=T)
rm(running.tally.A0.parallel)
gc()

# Save combined chain and model
save(running.tally.model,
     running.tally.A0,
     signlist,
     file="runningtallyposterior.RData", compress=T)

rm(running.tally.A0); gc()

####################################################################
# Midrange model
####################################################################

print(system.time(midrange.A0.parallel <- clusterApply(cl, 1:2,
                                                            parallel.model,
                                                            fit.model=midrange.model,
                                                            N1, N2, thin.iter)))

# Extracts the free parameters in the A0 matrix for analysis using
# convergence diagnostic tools

A0.free <- A0.chains2mcmc.list(midrange.A0.parallel)

# Convergence diagnostics

print(geweke.diag(A0.free))   # Geweke diagnostic
print(effectiveSize(A0.free))
print(gelman.diag(A0.free))

# Get the sign pattern for the A0 so we can normalize
signlist <- sign(apply(rbind(A0.free[[1]], A0.free[[2]]), 2, median))

# Save te A0 data for more analysis later
save(A0.free, file="A0free-midrange.RData", compress=T)
rm(A0.free); gc()

# Save the A0 object
midrange.A0 <- A0parallel2A0obj(midrange.A0.parallel)

# Save parallel chain
save(midrange.model,
     midrange.A0.parallel,
     signlist,
     file="A0parallel-midrange.RData", compress=T)
rm(midrange.A0.parallel)
gc()

# Save combined chain and model
save(midrange.model,
     midrange.A0,
     signlist,
     file="midrangeposterior.RData", compress=T)

rm(midrange.A0); gc()


####################################################################
# Best model
####################################################################

print(system.time(best.A0.parallel <- clusterApply(cl, 1:2,
                                                            parallel.model,
                                                            fit.model=best.model,
                                                            N1, N2, thin.iter)))

# Extracts the free parameters in the A0 matrix for analysis using
# convergence diagnostic tools

A0.free <- A0.chains2mcmc.list(best.A0.parallel)

# Convergence diagnostics

print(geweke.diag(A0.free))   # Geweke diagnostic
print(effectiveSize(A0.free))
print(gelman.diag(A0.free))

# Get the sign pattern for the A0 so we can normalize
signlist <- sign(apply(rbind(A0.free[[1]], A0.free[[2]]), 2, median))

# Save te A0 data for more analysis later
save(A0.free, file="A0free-best.RData", compress=T)
rm(A0.free); gc()

# Save the A0 object
best.A0 <- A0parallel2A0obj(best.A0.parallel)

# Save parallel chain
save(best.model,
     best.A0.parallel,
     signlist,
     file="A0parallel-best.RData", compress=T)
rm(best.A0.parallel)
gc()

# Save combined chain and model
save(best.model,
     best.A0,
     signlist,
     file="bestposterior.RData", compress=T)

rm(best.A0); gc()


####################################################################
# Best ML model
####################################################################

print(system.time(bestML.A0.parallel <- clusterApply(cl, 1:2,
                                                   parallel.model,
                                                   fit.model=bestML.model,
                                                   N1, N2, thin.iter)))

# Extracts the free parameters in the A0 matrix for analysis using
# convergence diagnostic tools

A0.free <- A0.chains2mcmc.list(bestML.A0.parallel)

# Convergence diagnostics

print(geweke.diag(A0.free))   # Geweke diagnostic
print(effectiveSize(A0.free))
print(gelman.diag(A0.free))

# Get the sign pattern for the A0 so we can normalize
signlist <- sign(apply(rbind(A0.free[[1]], A0.free[[2]]), 2, median))

# Save te A0 data for more analysis later
save(A0.free, file="A0free-bestML.RData", compress=T)
rm(A0.free); gc()

# Save the A0 object
bestML.A0 <- A0parallel2A0obj(bestML.A0.parallel)

# Save parallel chain
save(bestML.model,
     bestML.A0.parallel,
     signlist,
     file="A0parallel-bestML.RData", compress=T)
rm(bestML.A0.parallel)
gc()

# Save combined chain and model
save(bestML.model,
     bestML.A0,
     signlist,
     file="bestMLposterior.RData", compress=T)

rm(bestML.A0); gc()

####################################################################
# Flat prior model
####################################################################

print(system.time(flatprior.A0.parallel <- clusterApply(cl, 1:2,
                                                            parallel.model,
                                                            fit.model=flatprior.model,
                                                            N1, N2, thin.iter)))

# Extracts the free parameters in the A0 matrix for analysis using
# convergence diagnostic tools

A0.free <- A0.chains2mcmc.list(flatprior.A0.parallel)

# Convergence diagnostics

print(geweke.diag(A0.free))   # Geweke diagnostic
print(effectiveSize(A0.free))
print(gelman.diag(A0.free))

# Get the sign pattern for the A0 so we can normalize
signlist <- sign(apply(rbind(A0.free[[1]], A0.free[[2]]), 2, median))

# Save te A0 data for more analysis later
save(A0.free, file="A0free-flatprior.RData", compress=T)
rm(A0.free); gc()

# Save the A0 object
flatprior.A0 <- A0parallel2A0obj(flatprior.A0.parallel)

# Save parallel chain
save(flatprior.model,
     flatprior.A0.parallel,
     signlist,
     file="A0parallel-flatprior.RData", compress=T)
rm(flatprior.A0.parallel)
gc()

# Save combined chain and model
save(flatprior.model,
     flatprior.A0,
     signlist,
     file="flatpriorposterior.RData", compress=T)

rm(flatprior.A0); gc()


# Quit
mpi.quit(save="no")
